import { useQuery } from '@apollo/client'
import { StatusBar } from 'expo-status-bar'
import React, { memo, useLayoutEffect, useState } from 'react'
import { Trans, useTranslation } from 'react-i18next'
import { ImageBackground, Platform, StyleSheet, View } from 'react-native'

import ScrollHeaderLayout from '@holi/core/components/ScrollHeaderLayout'
import { logError } from '@holi/core/errors/helpers'
import { useBreakpointMatcher } from '@holi/core/helpers'
import { useOnRefresh } from '@holi/core/helpers/useOnRefresh'
import createParam from '@holi/core/navigation/hooks/useParam'
import { useUnifiedNavigation } from '@holi/core/navigation/hooks/useUnifiedNavigation'
import { InsightByIdResponse, insightById } from '@holi/core/screens/insights/queries'
import PostPreview from '@holi/core/screens/posts/components/PostPreview'
import { Post } from '@holi/core/screens/posts/types'
import { User } from '@holi/core/screens/userprofile/types'
import HoliGridCell from '@holi/ui/components/atoms/HoliGridCell'
import HoliGridWrapper from '@holi/ui/components/atoms/HoliGridWrapper'
import HoliHeading from '@holi/ui/components/atoms/HoliHeading'
import HoliText from '@holi/ui/components/atoms/HoliText'
import HoliAvatar from '@holi/ui/components/molecules/HoliAvatar'
import HoliButton from '@holi/ui/components/molecules/HoliButton'
import HoliModal from '@holi/ui/components/molecules/HoliModal'
import HoliUserCardsModalContent from '@holi/ui/components/organisms/HoliUserCardsModalContent'
import { colors } from '@holi/ui/styles/globalVars'
import { useStyleMixins } from '@holi/ui/styles/theme/mixins'

export type InsightDetailPageParams = {
  insightId: string
}

const { useParam } = createParam<InsightDetailPageParams>()

const InsightDetailPage = () => {
  const [insightId = ''] = useParam('insightId')
  const { t } = useTranslation()
  const mixins = useStyleMixins()
  const postGridItemsInRow = useBreakpointMatcher()
  const [showCuratorModal, setShowCuratorModal] = useState(false)
  const { setOptions } = useUnifiedNavigation()
  const { loading, error, data, refetch } = useQuery<InsightByIdResponse>(insightById, {
    variables: { id: insightId },
    notifyOnNetworkStatusChange: true,
  })

  const insight = data?.insightById

  const { refreshing, onRefresh } = useOnRefresh(refetch)

  useLayoutEffect(
    () =>
      setOptions({
        title: t('insights.insight.title', { title: insight?.title }),
        headerTransparent: true,
        headerTintColor: colors.background10,
        navigationCustomOptions: {
          headerBackgroundGradient: true,
          footerShown: false,
        },
      }),
    [setOptions, t, insight?.title]
  )

  if (error) logError(error, `[InsightDetailPage.tsx] ${error.message}`)

  if (!data && loading) return null

  if (!insight) return <HoliText>{t('insights.insight.notFound', { insightId })}</HoliText>

  const posts: Post[] = insight.posts || []
  const author = insight.creator

  return (
    <>
      <StatusBar style="light" animated />

      <ScrollHeaderLayout.Refreshable
        refreshableProps={{
          isLoading: refreshing,
          onRefresh,
        }}
        style={styles.wrapper}
        headerOverlap
      >
        {/* Image section */}
        <ImageBackground
          source={{ uri: insight?.headerImage }}
          resizeMode="cover"
          style={styles.insightBackgroundImage}
          imageStyle={styles.insightImage}
        />

        {/* Insight content */}
        <View style={[mixins.container, styles.insightView]}>
          <HoliHeading size="xSmall" style={styles.categoryHeadline} testID="insight-category">
            {insight?.topic?.title}
          </HoliHeading>

          <HoliHeading level="1" size="super" style={styles.insightHeadline} testID="insight-headline">
            {insight.title}
          </HoliHeading>

          <HoliText style={styles.insightText}>{insight.description}</HoliText>

          {/* Author  */}
          {author && (
            <View style={styles.authorInfo}>
              <HoliButton
                transparent
                label={t('user.cards.modal.showInfo', { name: author.fullName })}
                onPress={() => setShowCuratorModal(true)}
              >
                <View style={styles.authorInfoInner}>
                  <HoliAvatar
                    imageSrc={author.avatar}
                    label={author.fullName}
                    initials={author.avatarLabel || ''}
                    defaultColor={author.avatarDefaultColor || ''}
                    size="small"
                    style={styles.authorInfoAvatar}
                  />

                  <HoliText size="small" style={styles.authorInfoText}>
                    <Trans t={t} i18nKey="insight.curatedBy" values={{ name: author.fullName }}>
                      <HoliText size="small" />
                    </Trans>
                  </HoliText>
                </View>
              </HoliButton>
            </View>
          )}

          {author && (
            <HoliModal
              label={t('user.cards.modal.showInfo', { name: author.fullName })}
              show={showCuratorModal}
              onDismiss={() => setShowCuratorModal(false)}
              testID="insight-curator-modal"
            >
              <HoliUserCardsModalContent
                title={''} // don't show any headline in modal
                users={[author as User]}
                onDismiss={() => setShowCuratorModal(false)}
                testID="insight-curator"
              />
            </HoliModal>
          )}
        </View>

        {!!posts.length && (
          <View style={[mixins.container, styles.postList]}>
            <HoliGridWrapper itemsInRow={postGridItemsInRow}>
              {
                /** FIXME convert to VirtualizedList, FlatList or SectionList */
                posts.map((post) => (
                  <HoliGridCell key={post.id}>
                    <PostPreview post={post} insightId={insightId} />
                  </HoliGridCell>
                ))
              }
            </HoliGridWrapper>
          </View>
        )}
      </ScrollHeaderLayout.Refreshable>
    </>
  )
}

const styles = StyleSheet.create({
  wrapper: {
    backgroundColor: colors.background20,
    ...Platform.select({ web: { flex: 1 } }),
  },
  insightBackgroundImage: {
    height: Platform.OS === 'web' ? 680 : 340,
    ...Platform.select({
      web: {
        maxHeight: '50vh',
        objectFit: 'cover',
      },
      native: { maxHeight: '50%' },
    }),
  },
  insightImage: {
    ...Platform.select({ web: { height: 'auto' } }),
  },
  insightView: {
    backgroundColor: colors.background10,
    borderBottomLeftRadius: 24,
    borderBottomRightRadius: 24,
    paddingTop: 40,
    paddingBottom: 24,
    ...Platform.select({ web: { paddingHorizontal: 20 } }),
  },
  categoryHeadline: {
    color: colors.brandColorB30,
  },
  insightHeadline: {
    marginTop: 8,
  },
  insightText: {
    color: colors.foreground20,
    marginTop: 24,
  },
  authorInfo: {
    marginTop: 24,
  },
  authorInfoInner: {
    flexDirection: 'row',
    justifyContent: 'center',
    alignItems: 'center',
  },
  authorInfoAvatar: {
    marginRight: 16,
  },
  authorInfoText: {
    color: colors.foreground20,
  },
  authorInfoName: {
    color: colors.foreground30,
  },
  postList: {
    marginTop: 24,
  },
})

export default memo(InsightDetailPage)
